#property strict

#include <MyInclude\Statement.mqh>
#include <MyInclude\String.mqh>
#include <MyInclude\Web.mqh>

class MYFXBOOK
{
private:
  const string Password;

  static string CommentToString( const string comment )
  {
    return("\"" + STRING::ReplaceSubString(comment, "\"", "'")+ "\"");
  }

  static string OrderToString( const ORDER* Order, const string Delimeter = "," )
  {
    return((string)Order.Ticket             + Delimeter +
           IntegerToString(Order.OpenTime)  + Delimeter +
           (string)Order.Type               + Delimeter +
           (string)Order.Lots               + Delimeter +
                   Order.symbol             + Delimeter +
           (string)Order.OpenPrice          + Delimeter +
           (string)Order.StopLoss           + Delimeter +
           (string)Order.TakeProfit         + Delimeter +
           IntegerToString(Order.CloseTime) + Delimeter +
           (string)Order.ClosePrice         + Delimeter +
           (string)Order.Commission         + Delimeter +
           (string)Order.Swap               + Delimeter +
           (string)Order.Profit             + Delimeter +
           CommentToString(Order.comment)   + Delimeter +
           (string)Order.MagicNumber        + "\r\n");
  }

  static string AccountToString( const STATEMENT &Statement, const string Delimeter = "," )
  {
    return((string)Statement.GetNumber()   + Delimeter +
                   Statement.GetCurrency() + Delimeter +
           (string)Statement.GetBalance()  + Delimeter +
           (string)Statement.GetEquity()   + Delimeter +
           IntegerToString(TimeCurrent())  + Delimeter + "end\r\n");
  }

  static string StatementToString( const STATEMENT &Statement )
  {
    string Str = "";
    const int Amount = Statement.GetAmount();

    for (int i = 0; i < Amount; i++)
      Str = Str + OrderToString(Statement[i]);

    Str = Str + AccountToString(Statement);

    return(Str);
  }

  int Send( const string Statement, const string Name ) const
  {
    string Result, ResponseHeaders;
    const string PostURL = ".myfxbook.com/upload-auto.html?email=" + Login + "&pass=" + Password + "&id=" + Name + "&new=true";
    string URLS[2];

    URLS[0] = "http://upload1" + PostURL;
    URLS[1] = "http://upload2" + PostURL;

    for (int i = 0; i < ArraySize(URLS); i++)
      if (WEB::Request("POST", URLS[i], NULL, NULL, 10000, Statement, Result, ResponseHeaders) != -1)
        if ((StringToInteger(Result) == 200) || (StringToInteger(Result) == 401))
          break;

    return((int)StringToInteger(Result));
  }

  static string AnswerCodeToString( const int Code )
  {
    string Res;

    switch (Code)
    {
    case 200:
      Res = "Myfxbook - Statement uploaded successfully";

      break;
    case 401:
      Res = "Myfxbook - Wrong login details";

      break;
    case 503:
      Res = "Myfxbook servers are down for maintenance - please try again later";

      break;
    default:
      Res = "Myfxbook-unknown code: " + (string)Code;
    }

    return(Res);
  }

public:
  const string Login;

  MYFXBOOK( const string login, const string password) : Login(login), Password(STRING::StringToHEX(password))
  {
  }

  virtual void Publisher( const STATEMENT &Statement, string Name = "" )
  {
    Name = (string)StringToInteger(Name);

    if (StringToInteger(Name) <= 0)
      Name = (string)Statement.GetNumber();

    Print(AnswerCodeToString(Send(StatementToString(Statement), Name)));

    return;
  }
};