#property strict

#include <MyInclude\Pointer.mqh>
#include <MyInclude\File.mqh>

#define OP_DEPOSIT 6

#define CENT_DIGITS 2

#define EQUAL_SOURCE( VALUE )   VALUE = Source.VALUE;

#define ASSIGN_ORDER( VALUE )   VALUE = Order##VALUE();
#define ASSIGN_ACCOUNT( VALUE ) VALUE = Account##VALUE();

#define DEFINE_INT( VALUE )      int VALUE;
#define DEFINE_DATETIME( VALUE ) datetime VALUE;
#define DEFINE_DOUBLE( VALUE )   double VALUE;
#define DEFINE_STRING( VALUE )   string VALUE;

#define DEFINE_ORDER( T1, T2, T3 ) \
  T1(Ticket)                       \
  T1(Type)                         \
  T1(MagicNumber)                  \
  T2(OpenTime)                     \
  T2(CloseTime)                    \
  T2(Expiration)                   \
  T3(Lots)                         \
  T3(OpenPrice)                    \
  T3(ClosePrice)                   \
  T3(StopLoss)                     \
  T3(TakeProfit)                   \
  T3(Commission)                   \
  T3(Swap)                         \
  T3(Profit)

#define DEFINE_ACCOUNT( T1, T2, T3 ) \
  T1(Number)                         \
  T1(Leverage)                       \
  T2(Company)                        \
  T2(Server)                         \
  T2(Name)                           \
  T2(Currency)                       \
  T3(Balance)                        \
  T3(Equity)

#define FILE_WRITE_INT( VALUE )    FileWriteInteger(handle, VALUE);
#define FILE_WRITE_DOUBLE( VALUE ) FileWriteDouble(handle, VALUE);
#define FILE_WRITE_STRING( VALUE ) FILE::FileWriteStringBIN(handle, VALUE);
#define FILE_WRITE_DATETIME( VALUE ) FileWriteInteger(handle, (int)(VALUE));

#define FILE_READ_INT( VALUE )    VALUE = FileReadInteger(handle);
#define FILE_READ_DOUBLE( VALUE ) VALUE = FileReadDouble(handle);
#define FILE_READ_STRING( VALUE ) VALUE = FILE::FileReadStringBIN(handle);

#define DEFINE_GET_INT( VALUE )    \
int Get##VALUE( void ) const       \
{                                  \
  return(VALUE);                   \
}                                  \

#define DEFINE_GET_STRING( VALUE ) \
string Get##VALUE( void ) const    \
{                                  \
  return(VALUE);                   \
}                                  \

#define DEFINE_GET_DOUBLE( VALUE ) \
double Get##VALUE( void ) const    \
{                                  \
  return(VALUE);                   \
}                                  \

class ORDER
{
protected:
  void SetData( void )
  {
    DEFINE_ORDER(ASSIGN_ORDER, ASSIGN_ORDER, ASSIGN_ORDER)

    symbol = OrderSymbol();
    comment = OrderComment();

    return;
  }

  void Load( const int handle )
  {
    DEFINE_ORDER(FILE_READ_INT, FILE_READ_INT, FILE_READ_DOUBLE)

    symbol = FILE::FileReadStringBIN(handle);
    comment = FILE::FileReadStringBIN(handle);

    return;
  }

public:
  DEFINE_ORDER(DEFINE_INT, DEFINE_DATETIME, DEFINE_DOUBLE)

  string symbol;
  string comment;

  ORDER( const int handle = INVALID_HANDLE )
  {
    Init(handle);
  }

  virtual void Init( const int handle = INVALID_HANDLE )
  {
    if (handle == INVALID_HANDLE)
      SetData();
    else
      Load(handle);

    return;
  }

  void Save( const int handle ) const
  {
    DEFINE_ORDER(FILE_WRITE_INT, FILE_WRITE_DATETIME, FILE_WRITE_DOUBLE)

    FILE::FileWriteStringBIN(handle, symbol);
    FILE::FileWriteStringBIN(handle, comment);

    return;
  }

  void Zero( void )
  {
    ZeroMemory(this);

    return;
  }

  void SetDeposit( const double Deposit, const datetime time )
  {
    Zero();

    Type = OP_DEPOSIT;

    Profit = Deposit;

    OpenTime = time;
    CloseTime = time;

    return;
  }
};

class STATEMENT
{
private:
  void SetOrders( const int Total, const int Mode = MODE_TRADES )
  {
    for (int i = 0; i < Total; i++)
      if (OrderSelect(i, SELECT_BY_POS, Mode))
        AddOrder();

    return;
  }

  void SetData( void )
  {
    SetOrders(OrdersHistoryTotal(), MODE_HISTORY);
    SetOrders(OrdersTotal());

    DEFINE_ACCOUNT(ASSIGN_ACCOUNT, ASSIGN_ACCOUNT, ASSIGN_ACCOUNT)

    return;
  }

protected:
  ORDER* Orders[];
  int Amount;

  DEFINE_ACCOUNT(DEFINE_INT, DEFINE_STRING, DEFINE_DOUBLE)

  ORDER* AddOrder( const int handle = INVALID_HANDLE )
  {
    ArrayResize(Orders, Amount + 1);

    Orders[Amount] = new ORDER(handle);

    Amount++;

    return(Orders[Amount - 1]);
  }

  bool Load( const string FileName )
  {
    const int handle = FileOpen(FileName, FILE_READ | FILE_BIN);

    if (handle == INVALID_HANDLE)
      return(FALSE);

    DEFINE_ACCOUNT(FILE_READ_INT, FILE_READ_STRING, FILE_READ_DOUBLE)

    while (!FileIsEnding(handle))
      AddOrder(handle);

    FileClose(handle);

    return(TRUE);
  }

  double GetSumProfit( void ) const
  {
    double Sum = 0;

    for (int i = 0; i < Amount; i++)
      Sum += Orders[i].Profit + Orders[i].Swap + NormalizeDouble(Orders[i].Commission, CENT_DIGITS);

    return(Sum);
  }

  void Repair( void )
  {
    const double BeginBalance = NormalizeDouble(Equity - GetSumProfit(), CENT_DIGITS);

    if (BeginBalance != 0)
    {
      ORDER* TmpOrder = AddOrder();

      for (int i = Amount - 1; i > 0; i--)
        Orders[i] = Orders[i - 1];

      TmpOrder.SetDeposit(BeginBalance, Orders[0].OpenTime);

      Orders[0] = TmpOrder;
    }

    return;
  }

public:
  STATEMENT( const string FileName = "" )
  {
    Init(FileName);
  }

  ~STATEMENT( void )
  {
    POINTER::DeleteObject(Orders);
  }

  virtual void Init( const string FileName = "" )
  {
    POINTER::DeleteObject(Orders);

    Amount = 0;

    if (FileName == "")
      SetData();
    else
      Load(FileName);

    Repair();

    return;
  }

  const ORDER* operator []( const int Pos ) const
  {
    return(Orders[Pos]);
  }

  bool Save( const string FileName )
  {
    const int handle = FileOpen(FileName, FILE_WRITE | FILE_BIN);

    if (handle == INVALID_HANDLE)
      return(FALSE);

    DEFINE_ACCOUNT(FILE_WRITE_INT, FILE_WRITE_STRING, FILE_WRITE_DOUBLE)

    for (int i = 0; i < Amount; i++)
      Orders[i].Save(handle);

    FileClose(handle);

    return(TRUE);
  }

  template <typename T>
  void Filter( const T FilterRule )
  {
    return;
  }

  DEFINE_GET_INT(Amount)
  DEFINE_ACCOUNT(DEFINE_GET_INT, DEFINE_GET_STRING, DEFINE_GET_DOUBLE)
};